@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <plist/Data.h>
 
 namespace PList
@@ -38,7 +38,7 @@ Data::Data(const PList::Data& d) : Node(PLIST_DATA)
     plist_set_data_val(_node, &b[0], b.size());
 }
 
-Data& Data::operator=(PList::Data& b)
+Data& Data::operator=(const PList::Data& b)
 {
     plist_free(_node);
     _node = plist_copy(b.GetPlist());
@@ -70,10 +70,10 @@ std::vector<char> Data::GetValue() const
     uint64_t length = 0;
     plist_get_data_val(_node, &buff, &length);
     std::vector<char> ret(buff, buff + length);
-    free(buff);
+    delete buff;
     return ret;
 }
 
 
 
-};
+}  // namespace PList
