@@ -39,7 +39,7 @@ static void dictionary_fill(Dictionary *_this, std::map<std::string,Node*> &map,
         plist_dict_next_item(node, it, &key, &subnode);
         if (key && subnode)
             map[std::string(key)] = Node::FromPlist(subnode, _this);
-        free(key);
+        delete key;
     } while (subnode);
     free(it);
 }
@@ -62,7 +62,7 @@ Dictionary::Dictionary(const PList::Dictionary& d)
     dictionary_fill(this, _map, _node);
 }
 
-Dictionary& Dictionary::operator=(PList::Dictionary& d)
+Dictionary& Dictionary::operator=(const PList::Dictionary& d)
 {
     for (Dictionary::iterator it = _map.begin(); it != _map.end(); it++)
     {
@@ -99,21 +99,45 @@ Dictionary::iterator Dictionary::Begin()
     return _map.begin();
 }
 
+Dictionary::iterator Dictionary::begin()
+{
+    return _map.begin();
+}
+
 Dictionary::iterator Dictionary::End()
 {
     return _map.end();
 }
 
+Dictionary::iterator Dictionary::end()
+{
+    return _map.end();
+}
+
 Dictionary::const_iterator Dictionary::Begin() const
 {
     return _map.begin();
 }
 
+Dictionary::const_iterator Dictionary::begin() const
+{
+    return _map.begin();
+}
+
 Dictionary::const_iterator Dictionary::End() const
 {
     return _map.end();
 }
 
+Dictionary::const_iterator Dictionary::end() const
+{
+    return _map.end();
+}
+
+size_t Dictionary::size() const {
+    return _map.size();
+}
+
 Dictionary::iterator Dictionary::Find(const std::string& key)
 {
     return _map.find(key);
@@ -143,11 +167,6 @@ Dictionary::iterator Dictionary::Set(const std::string& key, const Node& node)
     return Set(key, &node);
 }
 
-Dictionary::iterator Dictionary::Insert(const std::string& key, Node* node)
-{
-    return this->Set(key, node);
-}
-
 void Dictionary::Remove(Node* node)
 {
     if (node)
@@ -156,7 +175,7 @@ void Dictionary::Remove(Node* node)
         plist_dict_get_item_key(node->GetPlist(), &key);
         plist_dict_remove_item(_node, key);
         std::string skey = key;
-        free(key);
+        delete key;
         _map.erase(skey);
         delete node;
     }
