@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <plist/Key.h>
 
 namespace PList
@@ -32,12 +32,12 @@ Key::Key(plist_t node, Node* parent) : Node(node, parent)
 {
 }
 
-Key::Key(const PList::Key& k) : Node(PLIST_UINT)
+Key::Key(const PList::Key& k) : Node(PLIST_INT)
 {
     plist_set_key_val(_node, k.GetValue().c_str());
 }
 
-Key& Key::operator=(PList::Key& k)
+Key& Key::operator=(const PList::Key& k)
 {
     plist_free(_node);
     _node = plist_copy(k.GetPlist());
@@ -67,14 +67,9 @@ std::string Key::GetValue() const
 {
     char* s = NULL;
     plist_get_key_val(_node, &s);
-    std::string ret;
-    if (s) {
-        ret = s;
-        free(s);
-    } else {
-        ret = "";
-    }
+    std::string ret = s ? s : "";
+    delete s;
     return ret;
 }
 
-};
+}  // namespace PList
